#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::Crop
#
# This class can be used to crop an image file via ImageMagick.
# =========================================================================== #
# require 'image_paradise/crop/crop.rb'
# ImageParadise::Crop.new(ARGV)
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class Crop < ::ImageParadise::Base # === ImageParadise::Crop

  # ========================================================================= #
  # === CONVERT_COMMAND
  # ========================================================================= #
  CONVERT_COMMAND = 'convert'

  # ========================================================================= #
  # === CROP_PARAMETERS_TO_USE
  #
  # These are the default parameters to be given to the crop-interface
  # of convert.
  # ========================================================================= #
  CROP_PARAMETERS_TO_USE = '200x200+25+35'

  # ========================================================================= #
  # === TEMP_DIRECTORY
  #
  # Here it is specified which directory is used for the creation
  # of the output-image file. This can be overruled by calling
  # set_output_file() with another argument.
  # ========================================================================= #
  TEMP_DIRECTORY = '/home/Temp/'

  # ========================================================================= #
  # === initialize
  #
  # This method only accepts two arguments. The first is the commandline
  # arguments.
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks given to this method next
    #
    # This must come after reset() has been called already.
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet_and_be_verbose
      # ===================================================================== #
      when :do_not_run_yet_and_be_verbose
        run_already = false
        @be_verbose = false
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      else
        # =================================================================== #
        # === Handles Hashes next
        # =================================================================== #
        if yielded.is_a? Hash
          # ================================================================= #
          # === :store_in_this_directory
          # ================================================================= #
          if yielded.has_key? :store_in_this_directory
            set_store_here(
              yielded.delete(:store_in_this_directory)
            )
          end
        end
      end
    end
    case run_already
    # ======================================================================= #
    # === :dont_run_yet
    # ======================================================================= #
    when :dont_run_yet,
         :do_not_run_yet
      run_already = false
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @temp_dir
    # ======================================================================= #
    if File.directory? TEMP_DIRECTORY
      @temp_directory = TEMP_DIRECTORY
    else
      @temp_directory = (Dir.pwd+'/').squeeze '/'
    end
    # ======================================================================= #
    # === @be_verbose
    # ======================================================================= #
    @be_verbose = true
    # ======================================================================= #
    # === @output_file
    #
    # We will use some dummy-default name initially. This will become
    # more correct if the user supplied a proper filename as the main
    # file to modify.
    # ======================================================================= #
    @output_file = 'output.png'
  end

  # ========================================================================= #
  # === output_file?
  #
  # This method will return the output-image file, which is the one that
  # will be generated on a successful run.
  # ========================================================================= #
  def output_file?
    @output_file
  end; alias result  output_file? # === result
       alias result? output_file? # === result?
       alias output? output_file? # === output?

  # ========================================================================= #
  # === set_temp_directory
  #
  # This method will designate the temp-directory in use.
  # ========================================================================= #
  def set_temp_directory(i)
    i << '/' unless i.end_with? '/'
    @temp_directory = i
  end; alias set_store_here set_temp_directory # === set_store_here

  # ========================================================================= #
  # === set_input_file
  #
  # Whenever this method is called, the output-filename will also be
  # determined, via the method called synchronize_output_filename().
  # ========================================================================= #
  def set_input_file(i = nil)
    i = i.first if i.is_a? Array
    @input_file = i
    synchronize_output_filename if i
  end; alias input_file= set_input_file # === input_file=

  # ========================================================================= #
  # === input_file?
  # ========================================================================= #
  def input_file?
    @input_file
  end

  # ========================================================================= #
  # === input_filename?
  #
  # This is guaranteed to be a String.
  # ========================================================================= #
  def input_filename?
    @input_file.to_s
  end; alias input? input_filename? # === input?

  # ========================================================================= #
  # === set_use_these_crop_parameters
  # ========================================================================= #
  def set_use_these_crop_parameters(
      i = :default
    )
    i = i.join(' ').strip if i.is_a? Array
    case i
    # ======================================================================= #
    # This is the default entry point here.
    # ======================================================================= #
    when nil,
         :default
      i = CROP_PARAMETERS_TO_USE
    end
    @use_these_crop_parameters = i
  end; alias set_crop_command    set_use_these_crop_parameters # === set_crop_command
       alias set_crop_parameters set_use_these_crop_parameters # === set_crop_parameters

  # ========================================================================= #
  # === crop_parameters?
  # ========================================================================= #
  def crop_parameters?
    @use_these_crop_parameters
  end

  # ========================================================================= #
  # === set_output_file
  #
  # Designate the output file via this method.
  # ========================================================================= #
  def set_output_file(
      i = :default
    )
    i = i.first if i.is_a? Array
    case i
    when nil,
        :default,
        :synchronize,
        :infer
      i = return_the_default_name_for_cropped_image
    end
    if i
      unless i.include? '/'
        i = File.absolute_path(i)
      end
    end
    @output_file = i
  end; alias output_file= set_output_file # === output_file=

  # ========================================================================= #
  # === temp_directory?
  # ========================================================================= #
  def temp_directory?
    @temp_directory
  end; alias temp_dir? temp_directory? # === temp_dir?

  # ========================================================================= #
  # === return_the_default_name_for_cropped_image
  # ========================================================================= #
  def return_the_default_name_for_cropped_image(
      i = input_file?
    )
    temp_directory?+
    'CROPPED_IMAGE_'+
    today+
    '_'+
    File.basename(i)
  end

  # ========================================================================= #
  # === run_the_system_command               (system tag, sys tag, crop tag)
  #
  # This method will perform the actual cropping-action, by delegation
  # towards "convert".
  #
  # The first input-argument should be the filename of the file that is
  # to be cropped, such as "foobar.png".
  #
  # The second input-argument should be the specific crop-parameters
  # that we are wanting to apply.
  #
  # A typical crop-argument in this regard looks like this:
  #
  #   -crop 40x30+40+30
  #
  # ========================================================================= #
  def run_the_system_command(
      input_filename            = input_filename?,
      use_these_crop_parameters = crop_parameters?,
      output_file               = output_file?
    )
    if input_filename.is_a? Array
      input_filename = input_filename.join(' ')
    end
    if use_these_crop_parameters.empty?
      use_these_crop_parameters = CROP_PARAMETERS_TO_USE
    end
    # ======================================================================= #
    # Next make sure that the parameters are a String.
    # ======================================================================= #
    use_these_crop_parameters = use_these_crop_parameters.to_s.dup
    # ======================================================================= #
    # The following code means that the user passed only one +5 rather
    # than two +5+5, hence we sanitize it here a little bit.
    # ======================================================================= #
    if use_these_crop_parameters.count('+') == 1
      use_these_crop_parameters << use_these_crop_parameters[
        use_these_crop_parameters.index('+')..-1
      ]
    end
    if use_these_crop_parameters.include? '%'
      use_these_crop_parameters << 'x+0+0' unless use_these_crop_parameters.start_with?('x')
    end
    # ======================================================================= #
    # In the past we synchronized this back, but since as of March 2021
    # we no longer do so.
    # ======================================================================= #
    #  set_crop_parameters(use_this_crop_command) # Synchronize it back here.
    # ======================================================================= #
    if be_verbose?
      opn; e 'Next cropping the image `'+sfancy(input_filename)+'`.'
    end
    _ = CONVERT_COMMAND+' '+
        input_filename+' -crop '+
        use_these_crop_parameters+
        ' '+
        output_file
    if File.exist? input_filename
      opn; esystem _
    else
      e 'No file exists at '+sfile(input_filename)
    end
    return output_file # It is also useful to return the file path of the generated file here.
  end; alias perform_crop_action     run_the_system_command # === perform_crop_action
       alias perform_the_crop_action run_the_system_command # === perform_the_crop_action
       alias do_crop                 run_the_system_command # === do_crop
       alias crop                    run_the_system_command # === crop

  # ========================================================================= #
  # === report_where_the_file_is_now_stored
  # ========================================================================= #
  def report_where_the_file_is_now_stored(
      be_verbose = be_verbose?
    )
    if be_verbose
      opn; e 'The output file should now be available at:'
      e
      e "  #{sfile(output_file?)}"
      e
    end
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      set_input_file(i.first)
    end
  end

  # ========================================================================= #
  # === synchronize_output_filename
  # ========================================================================= #
  def synchronize_output_filename
    set_output_file :synchronize
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    set_use_these_crop_parameters
    menu
    run_the_system_command
    report
  end

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    report_where_the_file_is_now_stored
  end

end

# =========================================================================== #
# === ImageParadise.crop
#
# Use this toplevel-method in order to crop a local image.
#
# The first argument to this method  is simply the local image file
# that you wish to crop, such as "foobar.png".
#
# If the second argument to this method is omitted then we will use
# the default settings for the used crop-parameters, which should
# be sufficiently useful enough in order to yield a cropped image.
# Otherwise, if a second argument is supplied, then these will
# be used as the crop-parameters for the current invocation run.
#
# The "-crop" operator of the ImageMagick suite also understands how
# to crop an image to just a percentage of its original size. Let's
# look at the following example, which would half the size of the
# given input-image at hand:
#
#   convert foo.png -crop 50%x+0+0 CROPPED_foo.png
#
# For a typical usage example for -crop, look at the following
# example:
#
#   convert -crop 1104x372+58+100 input.jpg output.jpg
#
# This means that the top-left corner is set at 58 pixels in,
# and 100 pixels down. Then, a rectangle will be used, with
# a width of 1104 pixels and a height of 372 pixel. That is
# the one that will be shown in the resulting, newly created
# image file.
#
# Specific usage examples for this API:
#
#   ImageParadise.crop('foo.png')
#   ImageParadise.crop('foo.png','40x30+10+10')
#   ImageParadise.crop('foo.png','40x30+10+10','bla.png')
#
# =========================================================================== #
# The official documentation for crop-related actions in regards to
# ImageMagick can be found here:
#
#   https://www.imagemagick.org/script/command-line-options.php?#crop
#
# =========================================================================== #
def self.crop(
      main_file, # The input-file is a mandatory argument.
      crop_parameters_to_use = :default,
      new_filename = :infer
    )
  _ = ImageParadise::Crop.new { :do_not_run_yet_and_be_verbose }
  _.set_input_file(main_file)
  _.set_crop_parameters(crop_parameters_to_use)
  _.set_output_file(new_filename)
  _.do_crop
  return _.output_file?
end; self.instance_eval { alias [] crop } # === ImageParadise::Crop[]

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::Crop.new(ARGV)
end # crop /Depot/NJOY/StringTime.jpg