#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
module ImageParadise

class ImageToAscii # === ImageParadise::ImageToAscii

  require 'uri'
  require 'open-uri'

  begin
    require 'rmagick' unless Object.const_defined? :Magick
  rescue LoadError; end

  # ========================================================================= #
  # === HTML
  # ========================================================================= #
  HTML = 'html'

  # ========================================================================= #
  # === BR
  # ========================================================================= #
  BR = '<br/>'

  attr_writer :image_chars

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      path_to_file,
      run_already = true
    )
    reset
    # open-uri open will fallback to IO open.
    path_to_file = [path_to_file] if path_to_file.is_a? String
    if path_to_file
      path_to_file.each {|path|
        open(path) { |file| @data = file.read }
      }
    end
    self
  end

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === ImageToAscii[]
  #
  # Usage example:
  #
  #   require 'image_to_ascii'
  #   ImageToAscii['screenshot_2014.png']
  #
  # ========================================================================= #
  def self.[](i)
    _ = ImageToAscii.new(i)
    _.to_ascii
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @data = nil
  end

  # ========================================================================= #
  # === to_ascii
  # ========================================================================= #
  def to_ascii(options = {})
    options = {
      width: 100, color: false, format: 'text'
    }.merge(options)

    img = Magick::Image.from_blob(@data).first

    width  = options[:width]
    scale  = (width.to_f / img.columns)
    height = ((img.rows * scale) / 2).to_i

    img.resize!(width, height)
    # ======================================================================= #
    # Check whether we wish to add colours.
    # ======================================================================= #
    color_image   = img.dup if options[:color]
    img           = img.quantize(
                      image_chars.length, Magick::GRAYColorspace
                    ).normalize
    quantum_calc  = Magick::QuantumRange / Magick::QuantumPixel.to_i
    image_chars.map! {|char|
      char == ' ' ? '&nbsp;' : char
    } if options[:format] == HTML

    border = "+#{'-' * width}+#{line_break(options[:format])}"
    border = html_char(border) if options[:format] == HTML

    output = border.dup

    img.view(0, 0, width, height) { |view|
      height.times { |i|
        output << '|'
        width.times { |j|
          character = image_chars[view[i][j].red/quantum_calc]
          if options[:format] == HTML
            if options[:color]
              pix = color_image.pixel_color(j,i)
              color_string = "color: #{pix.to_color( Magick::AllCompliance,false,8, true)};"
            else
              color_string = ''
            end
            character = html_char(character, color_string)
          else
            # text-format
            if options[:color]
              pix       = color_image.pixel_color(j,i)
              character = character.color(
                unified_rgb_value(pix.red),
                unified_rgb_value(pix.green),
                unified_rgb_value(pix.blue)
              )
            end
          end
          output << character
        }
        output << "|#{line_break(options[:format])}"
      }
    }
    @result = output + border
    return @result
  end

  # ========================================================================= #
  # === image_chars
  # ========================================================================= #
  def image_chars
    @image_chars ||= ' .~:+=o*x^%#@$MW'.chars.to_a
  end

  # ========================================================================= #
  # === inspect
  # ========================================================================= #
  def inspect
    "#<#{self.class.to_s}>"
  end

  # ========================================================================= #
  # === line_break
  # ========================================================================= #
  def line_break(format)
    (format == 'text') ? "\n" : BR
  end; private :line_break

  # ========================================================================= #
  # === unified_rgb_value
  # ========================================================================= #
  def unified_rgb_value(number)
    (Magick::QuantumDepth == 16) ? (number / 256) : number
  end; private :unified_rgb_value

  # ========================================================================= #
  # === html_char
  # ========================================================================= #
  def html_char(char, additional_style = '')
    "<span style=\"font-family: 'Lucida Console', Monaco, monospace; #{additional_style}\">#{char}</span>"
  end; private :html_char

end

# =========================================================================== #
# === ImageParadise.image_to_ascii
# =========================================================================== #
def self.image_to_ascii(i = ARGV)
  ImageParadise::ImageToAscii.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::ImageToAscii.new(ARGV)
end