#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::Optimizer
#
# This class will delegate towards optimizing e. g. image files.
#
# Usage example:
#
#   ImageParadise::Optimizer.new 'foo.png'
#   ImageParadise::Optimizer.new 'foo.jpg'
#
# =========================================================================== #
# require 'image_paradise/optimizer.rb'
# ImageParadise::Optimizer.new('foo.jpg')
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class Optimizer < Base # === ImageParadise::Optimizer

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === CMD_TO_RUN
  # ========================================================================= #
  CMD_TO_RUN = 'pngquant' # This is either pngquant or jpegoptim for now.

  # ========================================================================= #
  # === FILE
  # ========================================================================= #
  FILE = __FILE__

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    # ======================================================================= #
    # Handle Hash as input next. These is currently only used to disable
    # colours.
    # ======================================================================= #
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        # =================================================================== #
        # === :use_colours
        # =================================================================== #
        if yielded.has_key? :use_colours
          @use_colours = yielded.delete(:use_colours)
        end
        # =================================================================== #
        # === :may_we_exit
        # =================================================================== #
        if yielded.has_key? :may_we_exit
          set_may_we_exit(yielded.delete(:may_we_exit))
        end
      else
        case yielded
        when :do_not_exit
          @may_we_exit = false
        end
      end
    end
    set_these_files(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @may_we_exit
    # ======================================================================= #
    @may_we_exit = true
    # ======================================================================= #
    # === @output_file
    # ======================================================================= #
    @output_file = nil
    # ======================================================================= #
    # === @processed_n_files
    #
    # Keep track of how many image-files we already processed.
    # ======================================================================= #
    @processed_n_files = 0
    # ======================================================================= #
    # === @cmd_to_run
    # ======================================================================= #
    @cmd_to_run = CMD_TO_RUN
    # ======================================================================= #
    # === @use_colours
    # ======================================================================= #
    @use_colours = true
    # ======================================================================= #
    # === @obtain_this_image_format
    #
    # We can work either on .png files or on .jpg files.
    #
    # The following ivar will keep track of that.
    # ======================================================================= #
    @obtain_this_image_format = '.png'
  end

  # ========================================================================= #
  # === set_may_we_exit
  # ========================================================================= #
  def set_may_we_exit(i)
    case i
    when :do_not_exit
      i = false
    end
    @may_we_exit = i
  end

  # ========================================================================= #
  # === rds
  # ========================================================================= #
  def rds(i)
    i.squeeze('/')
  end

  # ========================================================================= #
  # === cmd_to_run?
  # ========================================================================= #
  def cmd_to_run?
    @cmd_to_run
  end; alias cmd? cmd_to_run? # === cmd

  # ========================================================================= #
  # === set_these_files
  #
  # We always have to work on an Array.
  # ========================================================================= #
  def set_these_files(i = 'ALL_PNG_IMAGES')
    if i.is_a? String
      i = i.to_s.dup
      i.strip!
      # ===================================================================== #
      # Assume that the user did input a String with internal ' '.
      # This means that we will split up this String and assume
      # that the entries will be valid files.
      # ===================================================================== #
      i = i.split(' ') if i.include? ' '
    end
    i = [i] unless i.is_a? Array # After this point, we have an array.
    menu(i)
    if i.any? {|_| _.include? 'localhost' }
      i.map! {|entry|
        _ = ENV['MY_DATA']+'/'
        entry = entry.gsub(/localhost/, _).
                      gsub(/^http:\/\//,'')
      }
    end
    @these_files = i
    sanitize_these_files # Always optimize it after the assignment.
  end

  # ========================================================================= #
  # === sanitize_these_files
  # ========================================================================= #
  def sanitize_these_files
    @these_files.map! {|entry|
      entry = entry.to_s.dup
      entry.strip!
      case entry # Next, act on a few special instructions. (case tag)
      # ===================================================================== #
      # === ALL_IMAGES
      # ===================================================================== #
      when 'ALL_IMAGES','ALL_PNG','ALLPNG','ALL_PNG_IMAGES','ALL',
           nil
        entry = Dir['*'+image_format?]
      # ===================================================================== #
      # === DEEP
      # ===================================================================== #
      when 'DEEP','ALL_SUBDIRS' # As above but includes all subdirectories.
        entry = Dir['**/*'+image_format?]
      end
      entry
    }
    @these_files.flatten!
  end

  # ========================================================================= #
  # === show_help_then_exit                                        (help tag)
  # ========================================================================= #
  def show_help_then_exit
    e
    Colours.ecomment '  ALL_IMAGES # Work on all images but only in the current directory.'
    Colours.ecomment '  DEEP       # Work on all images, including subdirectories.'
    Colours.ecomment '  OPEN       # Open this file here.'
    e
    exit
  end

  # ========================================================================= #
  # === open_this_file_here_then_exit
  # ========================================================================= #
  def open_this_file_here_then_exit
    _ = 'bluefish '+FILE
    opnn; e _; system _
    exit
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # These case-entries here are commandline-options, hence why we exit.
      # ===================================================================== #
      # === optimizer --open
      # ===================================================================== #
      when /-?-?open/,'OPEN'
        open_this_file_here_then_exit
      # ===================================================================== #
      # === optimizer --help
      # ===================================================================== #
      when /-?-?help/,'HELP','HELP?'
        show_help_then_exit unless File.exist?(i)
      end
    end
  end

  # ========================================================================= #
  # === these_files?
  # ========================================================================= #
  def these_files?
    @these_files
  end; alias files? these_files? # === files?
       alias input? these_files? # === input?

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    Colours.rev
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    return Colours.sfancy(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    return Colours.sfile(i)if use_colours?
    return i
  end

  # ========================================================================= #
  # === report_stats
  # ========================================================================= #
  def report_stats(i = file?)
    opnn
    e 'The old filesize of `'+sfile(i)+'` was: '+
       sfancy('%6s' % @old_filesize.to_s)
    opnn
    e 'The new filesize of `'+sfile(i)+'` is:  '+
       sfancy('%6s' % @new_filesize.to_s)
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === obtain_this_image_format?
  # ========================================================================= #
  def obtain_this_image_format?
    @obtain_this_image_format
  end; alias image_format? obtain_this_image_format? # === image_format?

  # ========================================================================= #
  # === is_png_file?
  # ========================================================================= #
  def is_png_file?(i) # check whether this file is a .png file or not.
    return true if File.extname(i) == image_format?
    false
  end; alias is_jpeg_file? is_png_file? # === is_jpeg_file?

  # ========================================================================= #
  # === set_old_filesize
  # ========================================================================= #
  def set_old_filesize(i = file?)
    if File.exist? i
      @old_filesize = File.size(i)
      determine_output_file(i)
    end
  end

  # ========================================================================= #
  # === rename_new_file_to_old_file
  # ========================================================================= #
  def rename_new_file_to_old_file(i = file?)
    FileUtils.mv(output?, i)
  end

  # ========================================================================= #
  # === determine_output_file
  #
  # This will determine the instance variable @output_file.
  # ========================================================================= #
  def determine_output_file(i)
    @output_file = i.dup # Will store here.
    @output_file[@output_file.index('.'),0] = '-fs8' # Append "-fs8".
  end

  # ========================================================================= #
  # === work_on_the_images
  # ========================================================================= #
  def work_on_the_images
    files?.each {|the_file|
      the_file = rds(the_file)
      extname = File.extname(the_file).delete('.')
      case extname
      # ===================================================================== #
      # === Handle .jpg or .jpeg files here
      # ===================================================================== #
      when 'jpg','jpeg'
        @processed_n_files += 1 # Increment the counter very early on.
        @cmd_to_run = 'jpegoptim'
        esystem @cmd_to_run+' '+the_file
      # ===================================================================== #
      # === Handle .png files here
      # ===================================================================== #
      when 'png'
        if File.exist? the_file
          @processed_n_files += 1 # Increment the counter very early on.
          if is_png_file?(the_file) # main tag
            set_old_filesize(the_file)
            run_sys_command(the_file)
            rename_new_file_to_old_file(the_file) # <-- Important for cleanup.
            report_stats(the_file)
          else
            opnn
            e 'The file '+sfile(the_file)+' is '+RED+'not'+rev+
              ' a '+image_format?+' file. We can not optimize it.'
          end
        else
          opnn
          e 'No file could be found at `'+sfile(the_file)+'`.'
        end
      else
        e 'Unsure what to do with `'+sfile(the_file)+'`.'
      end
    }
    consider_reporting_how_many_image_files_we_did_process
  end

  # ========================================================================= #
  # === output?
  # ========================================================================= #
  def output?
    @output_file
  end

  # ========================================================================= #
  # === red?
  # ========================================================================= #
  def red?
    Colours::RED
  end

  # ========================================================================= #
  # === consider_reporting_how_many_image_files_we_did_process
  # ========================================================================= #
  def consider_reporting_how_many_image_files_we_did_process
    if @processed_n_files > 1 # Only if we have at least processed 2 images.
      opnn
      e 'Finished! We processed '+sfancy(@processed_n_files.to_s)+' '+
        @obtain_this_image_format+' files.'
    end
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    Opn.opn(
      namespace: NAMESPACE, use_colours: @use_colours
    )
  end

  # ========================================================================= #
  # === run_sys_command
  #
  # The first argument should be an existing, local file such as
  # "foobar.jpg", without the quotes.
  # ========================================================================= #
  def run_sys_command(i)
    if i.include? "'"
      i = '"'+i+'"'
    end
    _ = cmd?.to_s.dup
    _ << ' --force' if i.include? 'png'
    _ << ' --quality=40-100 --strip --skip-if-larger --verbose'
    _ << " #{i}" # This is the filename here.
    opnn
    esystem(_)
    if File.exist? output? # Check whether the output file exists.
      @new_filesize = File.size(output?)
    else
      opnn; e "The file `#{sfile(output?)}` does not exist."
      if @may_we_exit
        opnn; e 'Thus exiting now.'
        exit
      end
    end
  end

  # ========================================================================= #
  # === check_whether_the_command_is_available_or_not
  #
  # We check whether the commands are available or not.
  # ========================================================================= #
  def check_whether_the_command_is_available_or_not
    result = `#{cmd?} 2>&1`
    is_available = true
    is_available = false if result.include? cmd?+': not found'
    if is_available == false
      opnn
      e 'The program `'+sfancy(cmd?)+'` is '+red?+'not'+rev+
        ' available. Please install it first.'
      if is_on_roebe?
        e 'As this is a roebe-system, we will try to make use of'
        e 'the RBT project in order to compile this program next.'
        try_to_compile_this_program(cmd?)
      end
      exit
    end
  end

  # ========================================================================= #
  # === try_to_compile_this_program
  #
  # This method will tab into the RBT project and compile the
  # given input at hand - which is typically "pngquant".
  #
  # The method was added mostly as a convenience-method, to
  # quickly install pngquant.
  # ========================================================================= #
  def try_to_compile_this_program(i = cmd?)
    require 'rbt'
    RBT::Compile.new(i)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    check_whether_the_command_is_available_or_not
    work_on_the_images
  end

  # ========================================================================= #
  # === ImageParadise::Optimizer[]
  # ========================================================================= #
  def self.[](i)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::Optimizer.new(ARGV)
end # optimizer