#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# The rotate command-line option of ImageMagick takes one argument, which
# is the number of degrees by which to rotate the image. A positive number
# is the number of degrees to the right, and a negative number is the
# number of degrees to the left.
#
# For specific rotate-actions you can look at the following website:
#
#   https://imagemagick.org/script/command-line-options.php#rotate
#
# The specific API would then be:
#
#   -rotate degrees{<}{>}
#
# =========================================================================== #
# require 'image_paradise/rotate/rotate.rb'
# ImageParadise.rotate_left(these_images, n_degrees, background_colour, output_file_here) 
# ImageParadise.rotate_right(these_images, n_degrees, background_colour, output_file_here)
# =========================================================================== #
module ImageParadise

  require 'image_paradise/toplevel_methods/esystem.rb'

  # ========================================================================= #
  # === ImageParadise.rotate_this_image
  #
  # This method will always operate on the first input argument(s) in
  # batch form - hence we convert the first argument given to an
  # Array, inside of the method. Doing so allows the user to provide
  # a bunch of images that will all receive the same rotate-operation.
  #
  # The second input-argument to this method specifies the degree for the
  # rotation; a value with a '+' or without any modified token means
  # "clockwise rotation", that is rotation towards the right. A value
  # that is negative here, aka has a leading '-', means "counter-clockwise
  # rotation", that is to the left. Thus, rotate-left is counter-clockwise;
  # and rotate-right is clockwise.
  #
  # When we use convert, the commandline flags should look like this:
  #
  #   convert -resize "600>" image.png Newimage/image.png
  #
  # ========================================================================= #
  def self.rotate_this_image(
      these_images               = ARGV,
      by_n_degrees               = '-45', # Default is -45.
      use_this_background_colour = nil,   # ← Fill with this background colour.
      output_file                = :infer
    )
    case by_n_degrees
    when :rotate_left,
         :left
      by_n_degrees = '-90'
    when :rotate_right,
         :right
      by_n_degrees = '+90'
    end
    these_images = [these_images].flatten.compact
    these_images.each {|this_image|
      case output_file
      when :infer, :default, nil
        output_file = 'new_file_'+File.basename(this_image)
      end
      # ===================================================================== #
      # Note that the -density 300 part was added in March 2021,
      # after I noticed a significant loss in quality after
      # rotate-operations.
      # ===================================================================== #
      _ = 'convert -density 300 -rotate "'+by_n_degrees.to_s+'"'.dup
      _ << " #{this_image}"
      if use_this_background_colour
        case use_this_background_colour.to_s
        when :default
          use_this_background_colour = nil
        when 'black'
          use_this_background_colour = 'FFFFFF'
        end
        if use_this_background_colour
          _ << ' -background "#'+use_this_background_colour.to_s+'"'
        end
      end
      _ << " #{output_file}"
      esystem _
    }
  end; self.instance_eval { alias rotate_these_images rotate_this_image } # === ImageParadise.rotate_these_images

  # ========================================================================= #
  # === ImageParadise.rotate_left
  #
  # The second argument specifies the degree-rotation.
  # ========================================================================= #
  def self.rotate_left(
      these_images               = ARGV,
      by_n_degrees               = :rotate_left,
      use_this_background_colour = nil,
      output_file                = :infer
    )
    ImageParadise.rotate_this_image(
      these_images, by_n_degrees, use_this_background_colour, output_file
    )
  end

  # ========================================================================= #
  # === ImageParadise.rotate_right
  # ========================================================================= #
  def self.rotate_right(
      these_images               = ARGV,
      by_n_degrees               = :rotate_right,
      use_this_background_colour = nil,
      output_file                = :infer
    )
    ImageParadise.rotate_this_image(
      these_images, by_n_degrees, use_this_background_colour, output_file
    )
  end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise.rotate_this_image(ARGV)
end # rotatethisimage /home/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/image_paradise/test/16x16_red_square_image_for_testing.png