#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::ImageBorder
#
# This class will create a border around a given image.
#
# The syntax should be:
#
#   image_border foobar.png --colour red --dimension 50x20
#
# Usage example:
#
#   ImageParadise::ImageBorder.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise/image_border.rb'
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class ImageBorder < ::ImageParadise::Base # === ImageParadise::ImageBorder.new

  # ========================================================================= #
  # === DELETE_OUTPUT_FILE_IF_IT_EXISTS
  # ========================================================================= #
  DELETE_OUTPUT_FILE_IF_IT_EXISTS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_this_image_file(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    set_border_colour :random # Initialize as default border-colour.
    @dimension = '10x10'
  end

  # ========================================================================= #
  # === colour?
  # ========================================================================= #
  def colour?
    @border_colour
  end

  # ========================================================================= #
  # === set_this_image_file
  # ========================================================================= #
  def set_this_image_file(i = '')
    i = i.join(' ').strip if i.is_a? Array
    i.squeeze!(' ')
    i = parse_this_input(i)
    i.strip!
    menu(i)
    @this_image_file = i
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    help_string = <<-EOF

These options are available:

  image_border foobar.png --colour RANDOM --dimension 50x25

EOF
    e help_string
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    case i # case tag
    # ======================================================================= #
    # === image_border --help
    # ======================================================================= #
    when /--help/
      show_help
      exit
    end
  end

  # ========================================================================= #
  # === parse_this_input
  #
  # This method will parse the commandline options.
  # ========================================================================= #
  def parse_this_input(i)
    if i =~ /(--colour (\S{1,25}))/
      set_colour($2.to_s.dup)
      i.sub!(/#{$1.to_s}/,'')
    end
    if i =~ /(--dimension (\S{1,25}))/
      set_dimension($2.to_s.dup)
      i.sub!(/#{$1.to_s}/,'')
    end
    return i
  end

  # ========================================================================= #
  # === set_dimension
  # ========================================================================= #
  def set_dimension(i)
    @dimension = i
  end

  # ========================================================================= #
  # === set_border_colour
  # ========================================================================= #
  def set_border_colour(i = 'SkyBlue')
    case i.to_s
    when 'default'
      i = 'black'
    when 'RANDOM','random'
      require 'colours' unless Object.const_defined? :Colours
      i = Colours.random_html_colour
    end
    @border_colour = i
  end; alias set_colour set_border_colour # === set_colour

  # ========================================================================= #
  # === this_image_file?
  # ========================================================================= #
  def this_image_file?
    @this_image_file
  end; alias input?      this_image_file? # === input?
       alias input_file? this_image_file? # === input_file?

  # ========================================================================= #
  # === name_of_the_output_file?
  # ========================================================================= #
  def name_of_the_output_file?
    'OUTPUT_'+input_file?.sub(/\.png$/,'').sub(/\.jpg$/,'')+'.png'
  end; alias output_file? name_of_the_output_file? # === output_file?

  # ========================================================================= #
  # === determine_name_of_the_output_file
  #
  # Use this method to determine the name of the output file that will
  # be created de-novo.
  # ========================================================================= #
  def determine_name_of_the_output_file
    _ = name_of_the_output_file?
    if File.exist?(_) and !DELETE_OUTPUT_FILE_IF_IT_EXISTS
      # ===================================================================== #
      # In this case use a randomized name.
      # ===================================================================== #
      sample = ''
      20.times { sample << ALPHABET.sample }
      _ = 'OUTPUT_'+sample+input_file?.sub(/\.png$/,'').sub(/\.jpg$/,'')+'.png'
    end
    return _
  end

  # ========================================================================= #
  # === cmd?
  # ========================================================================= #
  def cmd?
    @cmd
  end

  # ========================================================================= #
  # === run_main_command
  # ========================================================================= #
  def run_main_command
    _ = cmd?
    if DELETE_OUTPUT_FILE_IF_IT_EXISTS and output_file_exists?
      File.delete(output_file?)
    end
    e _
    system _
  end

  # ========================================================================= #
  # === assemble_main_command
  # ========================================================================= #
  def assemble_main_command
    @cmd = CONVERT+' '
    @cmd << input_file?+' '
    @cmd << "-bordercolor '#{colour?}' "
    @cmd << '-border '+@dimension+' '
    name_of_the_output_file = determine_name_of_the_output_file
    @cmd << name_of_the_output_file
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = input_file?
    if File.exist? _
      assemble_main_command
      run_main_command
    else
      opn; e 'No file called '+sfile(_)+' exists.'
    end
  end

  # ========================================================================= #
  # === ImageParadise::ImageBorder[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::ImageBorder.new(ARGV)
end # image_border foobar.png --colour RANDOM --dimension 50x25