#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'image_paradise/image_manipulations.rb'
# =========================================================================== #
require 'image_paradise/gm_support.rb'

module ImageParadise

class ImageManipulations
  
  # ========================================================================= #
  # === initialize
  #
  # The first argument to this method should be the path to the file
  # that we want to manipulate.
  # ========================================================================= #
  def initialize(i)
    reset
    set_file(i)
  end

  # ========================================================================= #
  # === set_file
  # ========================================================================= #
  def set_file(i)
    @file = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @filename_changed = false
    @display_sys_command = false
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end; alias file file? # === file

  # ========================================================================= #
  # === identify
  #
  # Runs ImageMagick's 'identify'.
  #
  # See http://www.imagemagick.org/script/identify.php
  # ========================================================================= #
  def identify(i = {})
    tokens = ['identify']
    tokens << expand_arguments_to_key_value_pairs(i) if i
    tokens << " '#{@file}#{"[#{i[:layer].to_s}]" if i[:layer]}'"
    tokens  = convert_to_command(tokens)
    output  = run_command(tokens).first
    output
  end

  # ========================================================================= #
  # === dimensions
  #
  # Return the x and y dimensions of an image, as a Hash.
  # ========================================================================= #
  def dimensions
    dimensions = identify( # Tap into Imagemagick's identify binary.
      layer: 0,
      format: '%wx%h'
    ).chomp.split('x')
    hash = {
      x: dimensions[0].to_i,
      y: dimensions[1].to_i
    }
    return hash
  end; alias width_and_height dimensions # === width_and_height

  # ========================================================================= #
  # === width
  #
  # Returns the x dimension of an image as an integer
  # ========================================================================= #
  def width
    dimensions[:x]
  end

  # ========================================================================= #
  # === height
  #
  # Returns the y dimension of an image as an integer
  # ========================================================================= #
  def height
    dimensions[:y]
  end

  # ========================================================================= #
  # === montage
  #
  # Runs ImageMagick's 'montage'.
  #
  # See http://www.imagemagick.org/script/montage.php
  # ========================================================================= #
  def montage(sources, i = {})
    tokens = ['montage']
    tokens << expand_arguments_to_key_value_pairs(i) if i
    sources.each {|source| tokens << " '#{source}'" }
    tokens << " '#{@file}'"
    tokens  = convert_to_command(tokens)
    success = run_command(tokens)[1]
    success
  end

  # ========================================================================= #
  # === filename_changed?
  # ========================================================================= #
  def filename_changed?
    @filename_changed
  end

  # ========================================================================= #
  # === do_manipulate
  #
  # Runs ImageMagick's 'mogrify' command. The first argument to this
  # method is an (optional) Hash.
  #
  # See http://www.imagemagick.org/script/mogrify.php
  #
  # Invocation example:
  #
  #   image.do_manipulate(scale: '80%') # => true
  #
  # ========================================================================= #
  def do_manipulate(i = {})
    tokens  = ['mogrify']
    tokens << expand_arguments_to_key_value_pairs(i) if i
    tokens << " '#{@file}#{"[#{i[:layer].to_s}]" if i[:layer]}'"
    tokens << " -annotate #{i[:annotate].to_s}" if i[:annotate]
    tokens  = convert_to_command(tokens)
    success = run_command(tokens)[1]
    replace_file(i[:format].to_s.downcase, i[:layer]) if success && i[:format]
    success
  end; alias manipulate! do_manipulate # === manipulate!

  # ========================================================================= #
  # === shrink
  #
  # Shrinks the given image at hand. Be careful with this method - it
  # will instantly shrink the given file, without keeping a backup. So
  # if you want to keep a backup, you have to copy the image file on
  # your own BEFORE invoking this method here.
  # ========================================================================= #
  def shrink(n_percent = '80%')
    unless n_percent.is_a? String
      n_percent = n_percent.to_s
    end
    n_percent << '%' unless n_percent.end_with? '%'
    do_manipulate(scale: n_percent)
  end

  # ========================================================================= #
  # === replace_file
  #
  # Replaces the old file (with the old file format) with the newly
  # generated one.
  #
  # The old file will be deleted and $file will be reset.
  #
  # If ImageMagick generated more than one file, $file will have a "*", 
  # so that all files generated will be manipulated in the following
  # steps.
  # ========================================================================= #
  def replace_file(format, layer)
    return if File.extname(@file) == format

    layer ||= 0
    layer = layer.split(',').first if layer.is_a? String

    File.delete(@file)

    @filename_changed = true

    path = File.join File.dirname(@file), File.basename(@file, File.extname(@file))
    new_file = find_file(path, format, layer)

    @file = new_file.call(path, format, '*') unless new_file.nil?
  end

  # ========================================================================= #
  # === find_file
  # ========================================================================= #
  def find_file(path, format, layer)
    possible_paths = [
      Proc.new { |inner_path, inner_format, _inner_layer|
        "#{inner_path}.#{inner_format}"
      },
      Proc.new { |inner_path, inner_format, inner_layer|
        "#{inner_path}-#{inner_layer}.#{format}"
      },
      Proc.new { |inner_path, inner_format, inner_layer|
        "#{inner_path}.#{inner_format}.#{inner_layer}"
      }
    ]
    possible_paths.find { |possible_path|
      File.exist?(possible_path.call(path, format, layer))
    }
  end

  # ========================================================================= #
  # === convert                                                 (convert tag)
  #
  # Runs ImageMagick's 'convert'.
  #
  # The second argument to this method has to be a Hash, if it is given.
  #
  # See http://www.imagemagick.org/script/convert.php
  # ========================================================================= #
  def convert(output, i = {})
    tokens  = ['convert']
    tokens << expand_arguments_to_key_value_pairs(i) if i
    tokens << " '#{@file}#{"[#{i[:layer].to_s}]" if i.has_key?(:layer)}'"
    tokens << " -annotate #{i[:annotate].to_s}" if i.has_key?(:annotate)
    tokens << " #{output}"
    tokens  = convert_to_command(tokens)
    success = run_command(tokens)[1]
    success
  end

  # ========================================================================= #
  # === convert_to_command
  #
  # This method adds a proper prefix, which is defined in the file
  # gm_support.rb. Then it joins the commands of the Array at hand.
  # ========================================================================= #
  def convert_to_command(tokens)
    # ======================================================================= #
    # Pass into the method prefix() next.
    # ======================================================================= #
    tokens[0] = prefix(tokens.first) if respond_to? :prefix
    tokens.flatten.join
  end

  # ========================================================================= #
  # === expand_arguments_to_key_value_pairs
  #
  # Convert the entries into a format that ImageMagick understands.
  # ========================================================================= #
  def expand_arguments_to_key_value_pairs(i)
    special_arguments = [ :layer, :annotate ]
    # ======================================================================= #
    # First, reject keys that are not :layer or :annotate.
    # ======================================================================= #
    i.reject {|key, _value| special_arguments.include?(key) }.map {|key, value|
      " -#{key} '#{value}'"
    }
  end; alias convert_arguments expand_arguments_to_key_value_pairs # === convert_arguments

  # ========================================================================= #
  # === run_command
  #
  # Use IO.popen to run the passed command at hand.
  #
  # The method returns an Array, holding the output, and the success
  # status.
  # ========================================================================= #
  def run_command(i)
    i = i.to_s
    if @display_sys_command
      # ===================================================================== #
      # Show the command if the ivar above is true.
      # ===================================================================== #
      e i
    end
    output = IO.popen(i) {|dataset| dataset.read }
    success = $?.exitstatus == 0 ? true : false
    [output, success]
  end

end

# =========================================================================== #
# === ImageParadise.identify
# =========================================================================== #
def self.identify(this_image, hash_arguments = {})
  ImageParadise::ImageManipulations.new(this_image).identify(hash_arguments)
end

# =========================================================================== #
# === ImageParadise.convert
# =========================================================================== #
def self.convert(this_image, hash_arguments = {})
  ImageParadise::ImageManipulations.new(this_image).convert(hash_arguments)
end

# =========================================================================== #
# === ImageParadise.dimensions
# =========================================================================== #
def self.dimensions(this_image, hash_arguments = {})
  ImageParadise::ImageManipulations.new(this_image).dimensions(hash_arguments)
end

# =========================================================================== #
# === ImageParadise.do_manipulate
# =========================================================================== #
def self.do_manipulate(this_image, hash_arguments = {})
  ImageParadise::ImageManipulations.new(this_image).do_manipulate(hash_arguments)
end

# =========================================================================== #
# === ImageParadise.shrink
# =========================================================================== #
def self.shrink(this_image, hash_arguments = {})
  ImageParadise::ImageManipulations.new(this_image).shrink(hash_arguments)
end

end