#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::MagicCardBorder
#
# This class will create a rounded card border over a given image file.
#
# Usage example:
#
#   ImageParadise::MagicCardBorder.new(ARGV)
#
# =========================================================================== #
# require 'image_paradise/utility_scripts/magic_card_border.rb'
# ImageParadise.:MagicCardBorder.new(ARGV)
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class MagicCardBorder < ::ImageParadise::Base # === ImageParadise::MagicCardBorder

  require 'image_paradise/toplevel_methods/misc.rb'
  require 'image_paradise/toplevel_methods/add_black_border_to_this_image.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      if yielded.is_a? Hash
        # =================================================================== #
        # === :output_file
        # =================================================================== #
        if yielded.has_key? :output_file
          set_output_file(yielded[:output_file])
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @n_pixels
    # ======================================================================= #
    @n_pixels = 10
    # ======================================================================= #
    # This specifies the default output file.
    # ======================================================================= #
    @output_file = :default
  end

  # ========================================================================= #
  # === set_n_pixels
  # ========================================================================= #
  def set_n_pixels(i)
    @n_pixels = i.to_i
  end; alias set_size set_n_pixels # === set_size

  # ========================================================================= #
  # === n_pixels?
  # ========================================================================= #
  def n_pixels?
    @n_pixels
  end

  # ========================================================================= #
  # === output_file?
  # ========================================================================= #
  def output_file?
    @output_file
  end

  # ========================================================================= #
  # === sanitize_the_name_of_the_output_file
  #
  # Note that this method will always apply File.absolute_path().
  # ========================================================================= #
  def sanitize_the_name_of_the_output_file_based_on_this_input(
      i = :default
    )
    case i
    when nil
      i = 'foobar.png'
    else
      # This also covers :default
      if i == :default
        i = 'foobar.png'
      end
      # ===================================================================== #
      # In this case we need to use the default.
      # ===================================================================== #
      i = File.basename("output_#{i}")
    end
    i = File.absolute_path(i)
    set_output_file(i)
  end

  # ========================================================================= #
  # === set_output_file
  #
  # This is simply a setter - it may not modify anything when invoked.
  # ========================================================================= #
  def set_output_file(i)
    @output_file = i
  end

  # ========================================================================= #
  # === show_help
  #
  # Invoke this method via:
  #
  #   magic-rounded --help
  #
  # ========================================================================= #
  def show_help
    e 'This class presently supports the following commandline options:'
    e
    e '  ----size=(n) # where n should be a number. This '\
      'specifies the size of the rounded border.'
    e
    e 'Example for this commandline flag:'
    e
    e ::Colours.tomato('  magic-rounded --size=5')
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments_via_leading_hyphens
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === magic-rounded --size=5
      #
      # Invocation example:
      #
      #   magic-rounded screenshot.png --size=10
      #
      # ===================================================================== #
      when /^-?-?size=(.+)$/i
        set_size($1.to_s.dup)
      # ===================================================================== #
      # === magic-rounded --help
      # ===================================================================== #
      when /^-?-?help$/i
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    i = commandline_arguments_as_stripped_string
    sanitize_the_name_of_the_output_file_based_on_this_input(i)
    if File.exist? i
      _ = ::ImageParadise.add_border_to_this_image(i, @n_pixels, 'black', @output_file)
      result = ::ImageParadise.rounded_corner(_, @output_file)
      return result
    else
      print ::Colours.rev
      opnn; e 'No file exists at `'+sfile(i)+'`.'
      return nil
    end
  end

end

# =========================================================================== #
# === ImageParadise.magic_card_border
#
# This method will try to achieve a card border style similar to
# the card game "magic the gathering".
#
# Usage example:
#
#   ImageParadise.magic_card_border 'foobar.jpg'
#
# =========================================================================== #
def self.magic_card_border(
   i           = ARGV,
   output_file = :default
  )
  _ = ::ImageParadise::MagicCardBorder.new(i) {{ output_file: output_file }}
  return _.output_file?
end

end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::MagicCardBorder.new(ARGV)
end # class_magic_card_border