#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ImageParadise::TextOnImage
#
# This class must be able to respond to commandline instructions such
# as:
#
#   ipa FOOBAR.png --text="Hello world" --colour=darkgreen --font-size=22 --position=3,4
#
# =========================================================================== #
# require 'image_paradise/utility_scripts/text_on_image.rb'
# ImageParadise::TextOnImage.new(ARGV)
# =========================================================================== #
require 'image_paradise/base/base.rb'

module ImageParadise

class TextOnImage < ::ImageParadise::Base # === ImageParadise::TextOnImage

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    reset
    set_commandline_arguments(i)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @namespace
    # ======================================================================= #
    @namespace = NAMESPACE
    # ======================================================================= #
    # === @internal_hash
    # ======================================================================= #
    @internal_hash = {}
    # ======================================================================= #
    # === :write_this_text
    #
    # This variable keeps track as to which text is to be written.
    # ======================================================================= #
    @internal_hash[:write_this_text] = ''.dup
    # ======================================================================= #
    # === :use_this_colour
    # ======================================================================= #
    @internal_hash[:use_this_colour] = nil
    # ======================================================================= #
    # === :strokewidth
    # ======================================================================= #
    @internal_hash[:strokewidth] = nil
    # ======================================================================= #
    # Set up some default values next past this point.
    # ======================================================================= #
    set_use_this_position
  end

  # ========================================================================= #
  # === determine_on_which_files_to_act
  # ========================================================================= #
  def determine_on_which_files_to_act
    @internal_hash[:work_on_these_files] = commandline_arguments?.select {|entry|
      File.exist?(entry)
    }
  end

  # ========================================================================= #
  # === notify_the_user_whether_any_file_could_be_found_or_not
  # ========================================================================= #
  def notify_the_user_whether_any_file_could_be_found_or_not
    _ = @internal_hash[:work_on_these_files]
    if _.empty?
      opnn; e 'No existing file could be found.'
    end
  end

  # ========================================================================= #
  # === colour_to_use?
  # ========================================================================= #
  def colour_to_use?
    _ = ::ImageParadise.default_colour?
    if @internal_hash[:use_this_colour]
      _ = @internal_hash[:use_this_colour]
    end
    return _.to_s
  end

  # ========================================================================= #
  # === set_use_this_position
  # ========================================================================= #
  def set_use_this_position(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = ::ImageParadise.default_position?
    end
    @internal_hash[:use_this_position] = i.to_s.dup
  end

  # ========================================================================= #
  # === coordinates?
  # ========================================================================= #
  def coordinates?
    @internal_hash[:use_this_position].to_s
  end

  # ========================================================================= #
  # === size?
  # ========================================================================= #
  def size?
    @internal_hash[:use_this_font_size].to_s
  end

  # ========================================================================= #
  # === set_use_this_text
  # ========================================================================= #
  def set_use_this_text(
      i, optional_change_stuff = nil
    )
    if i.is_a? Array
      case optional_change_stuff
      # ===================================================================== #
      # === :without_hyphened_arguments
      # ===================================================================== #
      when :without_hyphened_arguments
        i = i.reject {|entry| entry.start_with?('--') }
      end
      i = i.join(' ')
    end
    @internal_hash[:write_this_text] = i.to_s.dup
  end; alias set_text set_use_this_text # === set_text

  # ========================================================================= #
  # === text?
  # ========================================================================= #
  def text?
    @internal_hash[:write_this_text]
  end

  # ========================================================================= #
  # === parse_the_hyphened_commandline_arguments                   (menu tag)
  # ========================================================================= #
  def parse_the_hyphened_commandline_arguments(
      i = commandline_arguments?
    )
    html_colours = Colours.html_colours.map {|entry| Regexp.new("^-?-?#{entry}$") }
    i.select {|entry| entry.start_with? '--' }.each {|entry|
      case entry
      # ===================================================================== #
      # === --darkblue
      # ===================================================================== #
      when *html_colours
        set_use_this_colour(entry)
      # ===================================================================== #
      # === --strokewidth="abc def"
      # ===================================================================== #
      when /^-?-?strokewidth=(.+)/
        set_use_this_strokewidth($1)
      # ===================================================================== #
      # === --text="abc def"
      # ===================================================================== #
      when /^-?-?text=(.+)/
        set_use_this_text($1)
      # ===================================================================== #
      # === --colour=steelblue
      # ===================================================================== #
      when /^-?-?colou?r=(.+)/
        set_use_this_colour($1)
      # ===================================================================== #
      # === --font-size=3
      # ===================================================================== #
      when /^-?-?font(-|_)size=(.+)/ # We grab the second entry here.
        set_use_this_font_size($2)
      # ===================================================================== #
      # === --position=3
      # ===================================================================== #
      when /^-?-?position=(.+)/
        set_use_this_position($1)
      else
        if File.exist? entry
          # ================================================================= #
          # Existing local files will be captured here.
          # ================================================================= #
          set_filename(entry)
        end
      end
    }
    i.each {|entry| set_filename(entry) if File.exist?(entry) }
  end; alias menu parse_the_hyphened_commandline_arguments # === menu

  # ========================================================================= #
  # === set_use_this_strokewidth
  # ========================================================================= #
  def set_use_this_strokewidth(i)
    @internal_hash[:strokewidth] = i.to_s.dup
  end

  # ========================================================================= #
  # === strokewidth?
  # ========================================================================= #
  def strokewidth?
    @internal_hash[:strokewidth]
  end

  # ========================================================================= #
  # === set_files
  # ========================================================================= #
  def set_files(i)
    @internal_hash[:work_on_these_files] = [i].flatten.compact
  end; alias set_file     set_files # === set_file
       alias set_filename set_files # === set_filename

  # ========================================================================= #
  # === set_use_this_colour
  # ========================================================================= #
  def set_use_this_colour(i)
    @internal_hash[:use_this_colour] = i.to_s.dup.delete('-')
  end

  # ========================================================================= #
  # === use_default_colour
  # ========================================================================= #
  def use_default_colour
    ::ImageParadise.default_colour = :default
    set_use_this_colour(::ImageParadise.default_colour?)
  end

  # ========================================================================= #
  # === set_use_this_font_size
  # ========================================================================= #
  def set_use_this_font_size(i = :default)
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = 35
    end
    @internal_hash[:use_this_font_size] = i.to_s.dup.to_i
  end; alias set_font_size set_use_this_font_size # === set_font_size

  # ========================================================================= #
  # === do_process_the_given_files
  # ========================================================================= #
  def do_process_the_given_files(
      _ = @internal_hash[:work_on_these_files]
    )
    unless _.empty?
      _.each {|work_on_this_file|
        cmd_to_use = ''.dup
        cmd_to_use << 'convert '
        if size? and !size?.empty?
          cmd_to_use << "-pointsize #{size?} "
        end
        if colour_to_use? and !colour_to_use?.empty?
          cmd_to_use << "-fill #{colour_to_use?} "
        end
        # =================================================================== #
        # Next let's draw the text.
        # =================================================================== #
        cmd_to_use << '-draw "text '+coordinates?+" '"+text?+"'"+'" '
        cmd_to_use << "#{work_on_this_file} "
        if strokewidth?
          cmd_to_use << '-strokewidth '+strokewidth?.to_s+' '
        end
        cmd_to_use << 'output_file.png'
        esystem(cmd_to_use.squeeze(' '))
      }
    end if _
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    determine_on_which_files_to_act
    parse_the_hyphened_commandline_arguments
    notify_the_user_whether_any_file_could_be_found_or_not
    do_process_the_given_files
  end

end; end

if __FILE__ == $PROGRAM_NAME
  ImageParadise::TextOnImage.new(ARGV)
end # textonimage foo.jpg --text="Hello world" --colour=steelblue --font-size=55 --position=25,125
    # textonimage foo.jpg --text="Hello world" --colour=steelblue --font-size=55 --position=25,125 --strokewidth=10
    # textonimage foo.jpg --text="Hello world" --font-size=80 --position=35,125 --strokewidth=50
    # textonimage foo.jpg --text="Hello world" --colour=steelblue --font-size=55 --position=35,125 --strokewidth=50
